@@ -12,154 +12,28 @@ static int _debug = 1;	/* XXX if < 0 debugging, > 0 unusual error returns */
 
 #include <rpm/rpmtypes.h>
 #include <rpm/rpmmacro.h>
-#include <rpm/rpmfileutil.h>	/* rpmioMkPath */
+#include <rpm/rpmfileutil.h>
 #include <rpm/rpmlog.h>
 
 #include "lib/rpmdb_internal.h"
 
 #include "debug.h"
 
-#if !defined(DB_CLIENT)	/* XXX db-4.2.42 retrofit */
-#define	DB_CLIENT	DB_RPCCLIENT
-#endif
-
-
-/** \ingroup dbi
- * Hash database statistics.
- */
-struct dbiHStats_s {
-    unsigned int hash_magic;	/*!< hash database magic number. */
-    unsigned int hash_version;	/*!< version of the hash database. */
-    unsigned int hash_nkeys;	/*!< no. of unique keys in the database. */
-    unsigned int hash_ndata;	/*!< no. of key/data pairs in the database. */
-    unsigned int hash_pagesize;	/*!< db page (and bucket) size, in bytes. */
-    unsigned int hash_nelem;	/*!< estimated size of the hash table. */
-    unsigned int hash_ffactor;	/*!< no. of items per bucket. */
-    unsigned int hash_buckets;	/*!< no. of hash buckets. */
-    unsigned int hash_free;	/*!< no. of pages on the free list. */
-    unsigned int hash_bfree;	/*!< no. of bytes free on bucket pages. */
-    unsigned int hash_bigpages;	/*!< no. of big key/data pages. */
-    unsigned int hash_big_bfree;/*!< no. of bytes free on big item pages. */
-    unsigned int hash_overflows;/*!< no. of overflow pages. */
-    unsigned int hash_ovfl_free;/*!< no. of bytes free on overflow pages. */
-    unsigned int hash_dup;	/*!< no. of duplicate pages. */
-    unsigned int hash_dup_free;	/*!< no. bytes free on duplicate pages. */
-};
-
-/** \ingroup dbi
- * B-tree database statistics.
- */
-struct dbiBStats_s {
-    unsigned int bt_magic;	/*!< btree database magic. */
-    unsigned int bt_version;	/*!< version of the btree database. */
-    unsigned int bt_nkeys;	/*!< no. of unique keys in the database. */
-    unsigned int bt_ndata;	/*!< no. of key/data pairs in the database. */
-    unsigned int bt_pagesize;	/*!< database page size, in bytes. */
-    unsigned int bt_minkey;	/*!< minimum keys per page. */
-    unsigned int bt_re_len;	/*!< length of fixed-length records. */
-    unsigned int bt_re_pad;	/*!< padding byte for fixed-length records. */
-    unsigned int bt_levels;	/*!< no. of levels in the database. */
-    unsigned int bt_int_pg;	/*!< no. of database internal pages. */
-    unsigned int bt_leaf_pg;	/*!< no. of database leaf pages. */
-    unsigned int bt_dup_pg;	/*!< no. of database duplicate pages. */
-    unsigned int bt_over_pg;	/*!< no. of database overflow pages. */
-    unsigned int bt_free;	/*!< no. of pages on the free list. */
-    unsigned int bt_int_pgfree;	/*!< no. of bytes free in internal pages. */
-    unsigned int bt_leaf_pgfree;/*!< no. of bytes free in leaf pages. */
-    unsigned int bt_dup_pgfree;	/*!< no. of bytes free in duplicate pages. */
-    unsigned int bt_over_pgfree;/*!< no. of bytes free in overflow pages. */
-};
-
-#ifdef	NOTNOW
-static const char * bfstring(unsigned int x, const char * xbf)
-{
-    const char * s = xbf;
-    static char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
-    static char buf[256];
-    char * t, * te;
-    unsigned radix;
-    unsigned c, i, k;
-
-    radix = (s != NULL ? *s++ : 16);
-
-    if (radix <= 1 || radix >= 32)
-	radix = 16;
-
-    t = buf;
-    switch (radix) {
-    case 8:	*t++ = '0';	break;
-    case 16:	*t++ = '0';	*t++ = 'x';	break;
-    }
-
-    i = 0;
-    k = x;
-    do { i++; k /= radix; } while (k);
-
-    te = t + i;
-
-    k = x;
-    do { --i; t[i] = digits[k % radix]; k /= radix; } while (k);
-
-    t = te;
-    i = '<';
-    if (s != NULL)
-    while ((c = *s++) != '\0') {
-	if (c > ' ') continue;
-
-	k = (1 << (c - 1));
-	if (!(x & k)) continue;
-
-	if (t == te) *t++ = '=';
-
-	*t++ = i;
-	i = ',';
-	while (*s > ' ')
-	    *t++ = *s++;
-    }
-    if (t > te)	*t++ = '>';
-    *t = '\0';
-    return buf;
-}
-
-static const char * dbtFlags =
-	"\20\1APPMALLOC\2ISSET\3MALLOC\4PARTIAL\5REALLOC\6USERMEM\7DUPOK";
-
-static const char * dbenvOpenFlags =
-	"\20\1CREATE\2NO_EXCEPTIONS\3FORCE\4NOMMAP\5RDONLY\6RECOVER\7THREAD\10TXN_NOSYNC\11USE_ENVIRON\12USE_ENVIRON_ROOT\13CDB\14LOCK\15LOG\16MPOOL\17TXN\20JOINENV\21LOCKDOWN\22PRIVATE\23RECOVER_FATAL\24SYSTEM_MEM";
-
-static const char * dbOpenFlags =
-	"\20\1CREATE\2NO_EXCEPTIONS\3FORCE\4NOMMAP\5RDONLY\6RECOVER\7THREAD\10TXN_NOSYNC\11USE_ENVIRON\12USE_ENVIRON_ROOT\13EXCL\14FCNTL_LOCKING\15RDWRMASTER\16TRUNCATE\17EXTENT\20APPLY_LOGREG";
-
-static const char * dbenvSetFlags =
-	"\20\1CREATE\2NO_EXCEPTIONS\3FORCE\4NOMMAP\5RDONLY\6RECOVER\7THREAD\10TXN_NOSYNC\11USE_ENVIRON\12USE_ENVIRON_ROOT\13CDB_ALLDB\14NOLOCKING\15NOPANIC\16PANIC_ENV\17REGION_INIT\20YIELDCPU";
-
-static const char * dbSetFlags =
-	"\20\1DUP\2DUPSORT\3RECNUM\4RENUMBER\5REVSPLITOFF\6SNAPSHOT";
-
-static const char * dbiModeFlags =
-	"\20\1WRONLY\2RDWR\7CREAT\10EXCL\11NOCTTY\12TRUNC\13APPEND\14NONBLOCK\15SYNC\16ASYNC\17DIRECT\20LARGEFILE\21DIRECTORY\22NOFOLLOW";
-#endif	/* NOTNOW */
-
-
 static int cvtdberr(dbiIndex dbi, const char * msg, int error, int printit)
 {
-    int rc = error;
-
-    if (printit && rc) {
+    if (printit && error) {
+	int db_api = dbi->dbi_rpmdb->db_api;
 	if (msg)
 	    rpmlog(RPMLOG_ERR, _("db%d error(%d) from %s: %s\n"),
-		dbi->dbi_api, rc, msg, db_strerror(error));
+		db_api, error, msg, db_strerror(error));
 	else
 	    rpmlog(RPMLOG_ERR, _("db%d error(%d): %s\n"),
-		dbi->dbi_api, rc, db_strerror(error));
+		db_api, error, db_strerror(error));
     }
-
-    return rc;
+    return error;
 }
 
-static int db_fini(dbiIndex dbi, const char * dbhome,
-		const char * dbfile,
-		const char * dbsubfile)
+static int db_fini(dbiIndex dbi, const char * dbhome)
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
     DB_ENV * dbenv = rpmdb->db_dbenv;
@@ -171,9 +45,7 @@ static int db_fini(dbiIndex dbi, const char * dbhome,
     rc = dbenv->close(dbenv, 0);
     rc = cvtdberr(dbi, "dbenv->close", rc, _debug);
 
-    if (dbfile)
-	rpmlog(RPMLOG_DEBUG, "closed   db environment %s/%s\n",
-			dbhome, dbfile);
+    rpmlog(RPMLOG_DEBUG, "closed   db environment %s\n", dbhome);
 
     if (rpmdb->db_remove_env) {
 	int xx;
@@ -181,11 +53,10 @@ static int db_fini(dbiIndex dbi, const char * dbhome,
 	xx = db_env_create(&dbenv, 0);
 	xx = cvtdberr(dbi, "db_env_create", xx, _debug);
 	xx = dbenv->remove(dbenv, dbhome, 0);
-	xx = cvtdberr(dbi, "dbenv->remove", xx, _debug);
+	/* filter out EBUSY as it just means somebody else gets to clean it */
+	xx = cvtdberr(dbi, "dbenv->remove", xx, (xx == EBUSY ? 0 : _debug));
 
-	if (dbfile)
-	    rpmlog(RPMLOG_DEBUG, "removed  db environment %s/%s\n",
-			dbhome, dbfile);
+	rpmlog(RPMLOG_DEBUG, "removed  db environment %s\n", dbhome);
 
     }
     return rc;
@@ -218,15 +89,12 @@ static int db3isalive(DB_ENV *dbenv, pid_t pid, db_threadid_t tid, uint32_t flag
 }
 #endif
 
-static int db_init(dbiIndex dbi, const char * dbhome,
-		const char * dbfile,
-		const char * dbsubfile,
-		DB_ENV ** dbenvp)
+static int db_init(dbiIndex dbi, const char * dbhome, DB_ENV ** dbenvp)
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
     DB_ENV *dbenv = NULL;
     int eflags;
-    int rc;
+    int rc, xx;
 
     if (dbenvp == NULL)
 	return 1;
@@ -238,17 +106,11 @@ static int db_init(dbiIndex dbi, const char * dbhome,
     eflags = (dbi->dbi_oeflags | dbi->dbi_eflags);
     if (eflags & DB_JOINENV) eflags &= DB_JOINENV;
 
-    if (dbfile) {
-	char *dbiflags = prDbiOpenFlags(eflags, 1);
-	rpmlog(RPMLOG_DEBUG, "opening  db environment %s/%s %s\n",
-		dbhome, dbfile, dbiflags);
-	free(dbiflags);
+    {	char *fstr = prDbiOpenFlags(eflags, 1);
+	rpmlog(RPMLOG_DEBUG, "opening  db environment %s %s\n", dbhome, fstr);
+	free(fstr);
     }
 
-    /* XXX Can't do RPC w/o host. */
-    if (dbi->dbi_host == NULL)
-	dbi->dbi_ecflags &= ~DB_CLIENT;
-
     /* XXX Set a default shm_key. */
     if ((dbi->dbi_eflags & DB_SYSTEM_MEM) && dbi->dbi_shmkey == 0) {
 #if defined(HAVE_FTOK)
@@ -263,22 +125,11 @@ static int db_init(dbiIndex dbi, const char * dbhome,
     if (dbenv == NULL || rc)
 	goto errxit;
 
-  { int xx;
-
- /* 4.1: dbenv->set_app_dispatch(???) */
- /* 4.1: dbenv->set_alloc(???) */
- /* 4.1: dbenv->set_data_dir(???) */
- /* 4.1: dbenv->set_encrypt(???) */
-
+    dbenv->set_alloc(dbenv,rpmdb->db_malloc, rpmdb->db_realloc, rpmdb->db_free);
     dbenv->set_errcall(dbenv, (void *) rpmdb->db_errcall);
     dbenv->set_errfile(dbenv, rpmdb->db_errfile);
     dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
 
- /* 4.1: dbenv->set_feedback(???) */
- /* 4.1: dbenv->set_flags(???) */
-
- /* dbenv->set_paniccall(???) */
-
 #if (DB_VERSION_MAJOR >= 4 && DB_VERSION_MINOR >= 5)
     /* 
      * These enable automatic stale lock removal. 
@@ -288,75 +139,35 @@ static int db_init(dbiIndex dbi, const char * dbhome,
     dbenv->set_isalive(dbenv, db3isalive);
 #endif
 
-    if ((dbi->dbi_ecflags & DB_CLIENT) && dbi->dbi_host) {
-	const char * home;
-	int retry = 0;
-
-	if ((home = strrchr(dbhome, '/')) != NULL)
-	    dbhome = ++home;
-
-	while (retry++ < 5) {
-	    xx = dbenv->set_rpc_server(dbenv, NULL, dbi->dbi_host,
-		dbi->dbi_cl_timeout, dbi->dbi_sv_timeout, 0);
-	    xx = cvtdberr(dbi, "dbenv->set_server", xx, _debug);
-	    if (!xx)
-		break;
-	    (void) sleep(15);
-	}
-    } else {
 #if !(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
-		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
+    dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
+	    (dbi->dbi_verbose & DB_VERB_CHKPOINT));
 #endif
-	xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
-		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
-	xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
-		(dbi->dbi_verbose & DB_VERB_RECOVERY));
-	xx = dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
-		(dbi->dbi_verbose & DB_VERB_WAITSFOR));
+    dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
+	    (dbi->dbi_verbose & DB_VERB_DEADLOCK));
+    dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
+	    (dbi->dbi_verbose & DB_VERB_RECOVERY));
+    dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
+	    (dbi->dbi_verbose & DB_VERB_WAITSFOR));
 
-	if (dbi->dbi_mmapsize) {
-	    xx = dbenv->set_mp_mmapsize(dbenv, dbi->dbi_mmapsize);
-	    xx = cvtdberr(dbi, "dbenv->set_mp_mmapsize", xx, _debug);
-	}
-	if (dbi->dbi_tmpdir) {
-	    const char * root;
-	    char * tmpdir;
-
-	    root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
-	    if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
-		root = NULL;
-	    tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
-	    xx = dbenv->set_tmp_dir(dbenv, tmpdir);
-	    xx = cvtdberr(dbi, "dbenv->set_tmp_dir", xx, _debug);
-	    tmpdir = _free(tmpdir);
-	}
+    if (dbi->dbi_mmapsize) {
+	xx = dbenv->set_mp_mmapsize(dbenv, dbi->dbi_mmapsize);
+	xx = cvtdberr(dbi, "dbenv->set_mp_mmapsize", xx, _debug);
+    }
+    if (dbi->dbi_tmpdir) {
+	/* XXX this isn't correct wrt chroot in+out */
+	const char * root = rpmdb->db_chrootDone ? NULL : rpmdb->db_root;
+	char * tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
+	xx = dbenv->set_tmp_dir(dbenv, tmpdir);
+	xx = cvtdberr(dbi, "dbenv->set_tmp_dir", xx, _debug);
+	tmpdir = _free(tmpdir);
     }
-
- /* dbenv->set_lk_conflicts(???) */
- /* dbenv->set_lk_detect(???) */
- /* 4.1: dbenv->set_lk_max_lockers(???) */
- /* 4.1: dbenv->set_lk_max_locks(???) */
- /* 4.1: dbenv->set_lk_max_objects(???) */
-
- /* 4.1: dbenv->set_lg_bsize(???) */
- /* 4.1: dbenv->set_lg_dir(???) */
- /* 4.1: dbenv->set_lg_max(???) */
- /* 4.1: dbenv->set_lg_regionmax(???) */
 
     if (dbi->dbi_cachesize) {
 	xx = dbenv->set_cachesize(dbenv, 0, dbi->dbi_cachesize, 0);
 	xx = cvtdberr(dbi, "dbenv->set_cachesize", xx, _debug);
     }
 
- /* 4.1 dbenv->set_timeout(???) */
- /* dbenv->set_tx_max(???) */
- /* 4.1: dbenv->set_tx_timestamp(???) */
- /* dbenv->set_tx_recover(???) */
-
- /* dbenv->set_rep_transport(???) */
- /* dbenv->set_rep_limit(???) */
-
     if (dbi->dbi_no_fsync) {
 	xx = db_env_set_func_fsync(db3_fsync_disable);
 	xx = cvtdberr(dbi, "db_env_set_func_fsync", xx, _debug);
@@ -366,7 +177,6 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	xx = dbenv->set_shm_key(dbenv, dbi->dbi_shmkey);
 	xx = cvtdberr(dbi, "dbenv->set_shm_key", xx, _debug);
     }
-  }
 
     rc = (dbenv->open)(dbenv, dbhome, eflags, dbi->dbi_perms);
     rc = cvtdberr(dbi, "dbenv->open", rc, _debug);
@@ -398,13 +208,11 @@ static int db3sync(dbiIndex dbi, unsigned int flags)
 {
     DB * db = dbi->dbi_db;
     int rc = 0;
-    int _printit;
 
-    if (db != NULL)
+    if (db != NULL) {
 	rc = db->sync(db, flags);
-    /* XXX DB_INCOMPLETE is returned occaisionally with multiple access. */
-    _printit = _debug;
-    rc = cvtdberr(dbi, "db->sync", rc, _printit);
+	rc = cvtdberr(dbi, "db->sync", rc, _debug);
+    }
     return rc;
 }
 
@@ -579,39 +387,13 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
 static int db3close(dbiIndex dbi, unsigned int flags)
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
-    const char * root;
-    const char * home;
-    char * dbhome;
-    const char * dbfile;
-    const char * dbsubfile;
+    const char * dbhome = rpmdbHome(rpmdb);
     DB * db = dbi->dbi_db;
     int _printit;
     int rc = 0, xx;
 
     flags = 0;	/* XXX unused */
 
-    /*
-     * Get the prefix/root component and directory path.
-     */
-    root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
-    if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
-	root = NULL;
-    home = (dbi->dbi_home ? dbi->dbi_home : rpmdb->db_home);
-
-    dbhome = rpmGenPath(root, home, NULL);
-    if (dbi->dbi_temporary) {
-	dbfile = NULL;
-	dbsubfile = NULL;
-    } else {
-#ifdef	HACK	/* XXX necessary to support dbsubfile */
-	dbfile = (dbi->dbi_file ? dbi->dbi_file : db3basename);
-	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : rpmTagGetName(dbi->dbi_rpmtag));
-#else
-	dbfile = (dbi->dbi_file ? dbi->dbi_file : rpmTagGetName(dbi->dbi_rpmtag));
-	dbsubfile = NULL;
-#endif
-    }
-
     if (db) {
 	rc = db->close(db, 0);
 	/* XXX ignore not found error messages. */
@@ -620,13 +402,13 @@ static int db3close(dbiIndex dbi, unsigned int flags)
 	db = dbi->dbi_db = NULL;
 
 	rpmlog(RPMLOG_DEBUG, "closed   db index       %s/%s\n",
-		dbhome, (dbfile ? dbfile : rpmTagGetName(dbi->dbi_rpmtag)));
+		dbhome, dbi->dbi_file);
 
     }
 
     if (rpmdb->db_dbenv != NULL && dbi->dbi_use_dbenv) {
 	if (rpmdb->db_opens == 1) {
-	    xx = db_fini(dbi, (dbhome ? dbhome : ""), dbfile, dbsubfile);
+	    xx = db_fini(dbi, (dbhome ? dbhome : ""));
 	    rpmdb->db_dbenv = NULL;
 	}
 	rpmdb->db_opens--;
@@ -642,7 +424,6 @@ static int db3close(dbiIndex dbi, unsigned int flags)
 	dbenv->set_errcall(dbenv, (void *) rpmdb->db_errcall);
 	dbenv->set_errfile(dbenv, rpmdb->db_errfile);
 	dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
- /*	dbenv->set_paniccall(???) */
 #if !(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
 	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
@@ -655,6 +436,8 @@ static int db3close(dbiIndex dbi, unsigned int flags)
 		(dbi->dbi_verbose & DB_VERB_WAITSFOR));
 
 	if (dbi->dbi_tmpdir) {
+	    /* XXX this isn't correct wrt chroot in+out */
+	    const char * root = rpmdb->db_chrootDone ? NULL : rpmdb->db_root;
 	    char * tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
 	    rc = dbenv->set_tmp_dir(dbenv, tmpdir);
 	    rc = cvtdberr(dbi, "dbenv->set_tmp_dir", rc, _debug);
@@ -671,14 +454,13 @@ static int db3close(dbiIndex dbi, unsigned int flags)
 	rc = cvtdberr(dbi, "db_create", rc, _debug);
 
 	if (db != NULL) {
-		char * dbf = rpmGetPath(dbhome, "/", dbfile, NULL);
+		char * dbf = rpmGetPath(dbhome, "/", dbi->dbi_file, NULL);
 
 		rc = db->verify(db, dbf, NULL, NULL, flags);
 		rc = cvtdberr(dbi, "db->verify", rc, _debug);
 
 		rpmlog(RPMLOG_DEBUG, "verified db index       %s/%s\n",
-			(dbhome ? dbhome : ""),
-			(dbfile ? dbfile : rpmTagGetName(dbi->dbi_rpmtag)));
+			(dbhome ? dbhome : ""), dbi->dbi_file);
 
 	        /*
 		 * The DB handle may not be accessed again after
@@ -695,8 +477,6 @@ static int db3close(dbiIndex dbi, unsigned int flags)
 exit:
     dbi->dbi_db = NULL;
 
-    free(dbhome);
-
     dbi = db3Free(dbi);
 
     return rc;
@@ -705,11 +485,7 @@ exit:
 static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 {
     extern const struct _dbiVec db3vec;
-    const char * root;
-    const char * home;
-    char * dbhome;
-    const char * dbfile;
-    const char * dbsubfile;
+    const char *dbhome = rpmdbHome(rpmdb);
     dbiIndex dbi = NULL;
     int rc = 0;
     int xx;
@@ -728,29 +504,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      */
     if ((dbi = db3New(rpmdb, rpmtag)) == NULL)
 	return 1;
-    dbi->dbi_api = DB_VERSION_MAJOR;
-
-    /*
-     * Get the prefix/root component and directory path.
-     */
-    root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
-    if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
-	root = NULL;
-    home = (dbi->dbi_home ? dbi->dbi_home : rpmdb->db_home);
-
-    dbhome = rpmGenPath(root, home, NULL);
-    if (dbi->dbi_temporary) {
-	dbfile = NULL;
-	dbsubfile = NULL;
-    } else {
-#ifdef	HACK	/* XXX necessary to support dbsubfile */
-	dbfile = (dbi->dbi_file ? dbi->dbi_file : db3basename);
-	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : rpmTagGetName(dbi->dbi_rpmtag));
-#else
-	dbfile = (dbi->dbi_file ? dbi->dbi_file : rpmTagGetName(dbi->dbi_rpmtag));
-	dbsubfile = NULL;
-#endif
-    }
 
     oflags = (dbi->dbi_oeflags | dbi->dbi_oflags);
     oflags &= ~DB_TRUNCATE;	/* XXX this is dangerous */
@@ -778,11 +531,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 #endif
     }
 
-    /*
-     * Create the /var/lib/rpm directory if it doesn't exist (root only).
-     */
-    (void) rpmioMkpath(dbhome, 0755, getuid(), getgid());
-
     /*
      * Avoid incompatible DB_CREATE/DB_RDONLY flags on DBENV->open.
      */
@@ -842,8 +590,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      */
     if ((oflags & DB_CREATE) && (oflags & DB_RDONLY)) {
 	/* dbhome is writable, and DB->open flags may conflict. */
-	const char * dbfn = (dbfile ? dbfile : rpmTagGetName(dbi->dbi_rpmtag));
-	char * dbf = rpmGetPath(dbhome, "/", dbfn, NULL);
+	char * dbf = rpmGetPath(dbhome, "/", dbi->dbi_file, NULL);
 
 	if (access(dbf, F_OK) == -1) {
 	    /* File does not exist, DB->open might create ... */
@@ -870,7 +617,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 
     if (dbi->dbi_use_dbenv) {
 	if (rpmdb->db_dbenv == NULL) {
-	    rc = db_init(dbi, dbhome, dbfile, dbsubfile, &dbenv);
+	    rc = db_init(dbi, dbhome, &dbenv);
 	    if (rc == 0) {
 		rpmdb->db_dbenv = dbenv;
 		rpmdb->db_opens = 1;
@@ -883,8 +630,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 
     {	char *dbiflags = prDbiOpenFlags(oflags, 0);
     	rpmlog(RPMLOG_DEBUG, "opening  db index       %s/%s %s mode=0x%x\n",
-		dbhome, (dbfile ? dbfile : rpmTagGetName(dbi->dbi_rpmtag)),
-		dbiflags, dbi->dbi_mode);
+		dbhome, dbi->dbi_file, dbiflags, dbi->dbi_mode);
 	free(dbiflags);
     }
 
@@ -894,25 +640,16 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	rc = db_create(&db, dbenv, dbi->dbi_cflags);
 	rc = cvtdberr(dbi, "db_create", rc, _debug);
 	if (rc == 0 && db != NULL) {
-
-	    if (rc == 0 &&
-			rpmdb->db_malloc && rpmdb->db_realloc && rpmdb->db_free)
-	    {
+	    if (rc == 0 && !dbi->dbi_use_dbenv) {
 		rc = db->set_alloc(db,
 			rpmdb->db_malloc, rpmdb->db_realloc, rpmdb->db_free);
 		rc = cvtdberr(dbi, "db->set_alloc", rc, _debug);
 	    }
 
-/* 4.1: db->set_cache_priority(???) */
 	    if (rc == 0 && !dbi->dbi_use_dbenv && dbi->dbi_cachesize) {
 		rc = db->set_cachesize(db, 0, dbi->dbi_cachesize, 0);
 		rc = cvtdberr(dbi, "db->set_cachesize", rc, _debug);
 	    }
-/* 4.1: db->set_encrypt(???) */
-/* 4.1: db->set_errcall(dbenv, rpmdb->db_errcall); */
-/* 4.1: db->set_errfile(dbenv, rpmdb->db_errfile); */
-/* 4.1: db->set_errpfx(dbenv, rpmdb->db_errpfx); */
- /* 4.1: db->set_feedback(???) */
 
 	    if (rc == 0 && dbi->dbi_lorder) {
 		rc = db->set_lorder(db, dbi->dbi_lorder);
@@ -922,7 +659,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		rc = db->set_pagesize(db, dbi->dbi_pagesize);
 		rc = cvtdberr(dbi, "db->set_pagesize", rc, _debug);
 	    }
- /* 4.1: db->set_paniccall(???) */
 	    if (rc == 0 && oflags & DB_CREATE) {
 		switch(dbi->dbi_type) {
 		default:
@@ -954,7 +690,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		    }
 		    break;
 		case DB_BTREE:
-/* 4.1: db->set_append_recno(???) */
 		    if (dbi->dbi_bt_flags) {
 			rc = db->set_flags(db, dbi->dbi_bt_flags);
 			rc = cvtdberr(dbi, "db->set_bt_flags", rc, _debug);
@@ -983,7 +718,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		    break;
 		case DB_RECNO:
 		    if (dbi->dbi_re_delim) {
-/* 4.1: db->set_append_recno(???) */
 			rc = db->set_re_delim(db, dbi->dbi_re_delim);
 			rc = cvtdberr(dbi, "db->set_re_selim", rc, _debug);
 			if (rc) break;
@@ -1015,19 +749,14 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	    }
 
 	    if (rc == 0) {
-		char * fullpath;
-		const char * dbpath;
-		fullpath = rpmGetPath(dbhome, "/", dbfile ? dbfile : "", NULL);
-
-#ifdef	HACK	/* XXX necessary to support dbsubfile */
-		dbpath = (!dbi->dbi_use_dbenv && !dbi->dbi_temporary)
-			? fullpath : dbfile;
-#else
-		dbpath = (!dbi->dbi_temporary)
-			? fullpath : dbfile;
-#endif
-
-		rc = (db->open)(db, txnid, dbpath, dbsubfile,
+		const char *dbpath = dbi->dbi_temporary ? NULL : dbi->dbi_file;
+		char *fullpath = NULL;
+		/* When not in environment, absolute path is needed */
+		if (!dbi->dbi_use_dbenv && !dbi->dbi_temporary) {
+		    fullpath = rpmGetPath(dbhome, "/", dbi->dbi_file, NULL);
+		    dbpath = fullpath;
+		}
+		rc = (db->open)(db, txnid, dbpath, NULL,
 		    dbi->dbi_type, oflags, dbi->dbi_perms);
 
 		if (rc == 0 && dbi->dbi_type == DB_UNKNOWN) {
@@ -1064,7 +793,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	     * glibc/kernel combinations.
 	     */
 	    if (rc == 0 && dbi->dbi_lockdbfd &&
-		!((dbi->dbi_ecflags & DB_CLIENT) && dbi->dbi_host) &&
 		(!dbi->dbi_use_dbenv || _lockdbfd++ == 0))
 	    {
 		int fdno = -1;
@@ -1092,11 +820,11 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 				_("cannot get %s lock on %s/%s\n"),
 				((dbi->dbi_mode & O_ACCMODE) == O_RDONLY)
 					? _("shared") : _("exclusive"),
-				dbhome, (dbfile ? dbfile : ""));
-		    } else if (dbfile) {
+				dbhome, dbi->dbi_file);
+		    } else {
 			rpmlog(RPMLOG_DEBUG,
 				"locked   db index       %s/%s\n",
-				dbhome, dbfile);
+				dbhome, dbi->dbi_file);
 		    }
 		}
 	    }
@@ -1113,8 +841,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	(void) db3close(dbi, 0);
     }
 
-    free(dbhome);
-
     return rc;
 }
 
