@@ -235,12 +235,12 @@ static int findTag(headerSprintfArgs hsa, sprintfToken token, const char * name)
     stag->fmt = NULL;
     stag->tag = -1;
 
-    if (!strcmp(tagname, "*")) {
+    if (rstreq(tagname, "*")) {
 	stag->tag = -2;
 	goto bingo;
     }
 
-    if (strncmp("RPMTAG_", tagname, sizeof("RPMTAG_")-1) == 0) {
+    if (rstreqn("RPMTAG_", tagname, sizeof("RPMTAG_")-1)) {
 	tagname += sizeof("RPMTAG");
     }
 
@@ -289,20 +289,18 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
     sprintfToken format;
     sprintfToken token;
     int numTokens;
-    int i;
     int done = 0;
 
     /* upper limit on number of individual formats */
     numTokens = 0;
     if (str != NULL)
     for (chptr = str; *chptr != '\0'; chptr++)
-	if (*chptr == '%') numTokens++;
+	if (*chptr == '%' || *chptr == '[') numTokens++;
     numTokens = numTokens * 2 + 1;
 
     format = xcalloc(numTokens, sizeof(*format));
     if (endPtr) *endPtr = NULL;
 
-    /* LCL: can't detect done termination */
     dst = start = str;
     numTokens = 0;
     token = NULL;
@@ -330,10 +328,8 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 		char * newEnd;
 
 		start++;
-		if (parseExpression(hsa, token, start, &newEnd))
-		{
-		    format = freeFormat(format, numTokens);
-		    return 1;
+		if (parseExpression(hsa, token, start, &newEnd)) {
+		    goto errxit;
 		}
 		start = newEnd;
 		break;
@@ -346,8 +342,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    while (*chptr && *chptr != '{' && *chptr != '%') chptr++;
 	    if (!*chptr || *chptr == '%') {
 		hsa->errmsg = _("missing { after %");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
 	    *chptr++ = '\0';
@@ -369,8 +364,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    while (*next && *next != '}') next++;
 	    if (!*next) {
 		hsa->errmsg = _("missing } after %{");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 	    *next++ = '\0';
 
@@ -381,8 +375,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 		*chptr++ = '\0';
 		if (!*chptr) {
 		    hsa->errmsg = _("empty tag format");
-		    format = freeFormat(format, numTokens);
-		    return 1;
+		    goto errxit;
 		}
 		token->u.tag.type = chptr;
 	    } 
@@ -393,17 +386,14 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    
 	    if (!*start) {
 		hsa->errmsg = _("empty tag name");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
-	    i = 0;
 	    token->type = PTOK_TAG;
 
 	    if (findTag(hsa, token, start)) {
 		hsa->errmsg = _("unknown tag");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
 	    start = next;
@@ -417,16 +407,13 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    if (parseFormat(hsa, start,
 			    &token->u.array.format,
 			    &token->u.array.numTokens,
-			    &start, PARSER_IN_ARRAY))
-	    {
-		format = freeFormat(format, numTokens);
-		return 1;
+			    &start, PARSER_IN_ARRAY)) {
+		goto errxit;
 	    }
 
 	    if (!start) {
 		hsa->errmsg = _("] expected at end of array");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
 	    dst = start;
@@ -438,8 +425,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	case ']':
 	    if (state != PARSER_IN_ARRAY) {
 		hsa->errmsg = _("unexpected ]");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 	    *start++ = '\0';
 	    if (endPtr) *endPtr = start;
@@ -449,8 +435,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	case '}':
 	    if (state != PARSER_IN_EXPR) {
 		hsa->errmsg = _("unexpected }");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 	    *start++ = '\0';
 	    if (endPtr) *endPtr = start;
@@ -479,7 +464,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
     if (dst != NULL)
         *dst = '\0';
 
-    for (i = 0; i < numTokens; i++) {
+    for (int i = 0; i < numTokens; i++) {
 	token = format + i;
 	if (token->type == PTOK_STRING)
 	    token->u.string.len = strlen(token->u.string.string);
@@ -487,8 +472,11 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 
     *numTokensPtr = numTokens;
     *formatPtr = format;
-
     return 0;
+
+errxit:
+    freeFormat(format, numTokens);
+    return 1;
 }
 
 static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
@@ -655,8 +643,8 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 	t = hsaReserve(hsa, need);
 	te = stpcpy(t, val);
 	hsa->vallen += (te - t);
-	val = _free(val);
     }
+    free(val);
 
     return (hsa->val + hsa->vallen);
 }
@@ -701,7 +689,7 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 	break;
 
     case PTOK_COND:
-	if (getCached(hsa->cache, token->u.cond.tag.tag) ||
+	if (getData(hsa, token->u.cond.tag.tag) ||
 		      headerIsEntry(hsa->h, token->u.cond.tag.tag)) {
 	    spft = token->u.cond.ifFormat;
 	    condNumFormats = token->u.cond.numIfTokens;
@@ -761,7 +749,7 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 
 	    spft = token->u.array.format;
 	    isxml = (spft->type == PTOK_TAG && spft->u.tag.type != NULL &&
-		!strcmp(spft->u.tag.type, "xml"));
+		    rstreq(spft->u.tag.type, "xml"));
 
 	    if (isxml) {
 		const char * tagN = rpmTagGetName(spft->u.tag.tag);
@@ -854,7 +842,7 @@ char * headerFormat(Header h, const char * fmt, errmsg_t * errmsg)
 	(hsa.format->type == PTOK_ARRAY
 	    ? &hsa.format->u.array.format->u.tag :
 	NULL));
-    isxml = (tag != NULL && tag->tag == -2 && tag->type != NULL && !strcmp(tag->type, "xml"));
+    isxml = (tag != NULL && tag->tag == -2 && tag->type != NULL && rstreq(tag->type, "xml"));
 
     if (isxml) {
 	need = sizeof("<rpmHeader>\n") - 1;
