@@ -11,17 +11,10 @@
 
 #include "debug.h"
 
-/**
- * Enable noisy range comparison debugging message?
- */
-static int _noisy_range_comparison_debug_message = 0;
-
 int _rpmds_debug = 0;
 
 int _rpmds_nopromote = 1;
 
-int _rpmds_unspecified_epoch_noise = 0;
-
 /**
  * A package dependency set.
  */
@@ -146,8 +139,7 @@ rpmds rpmdsNew(Header h, rpmTag tagN, int flags)
 	goto exit;
 
     if (headerGet(h, tagN, &names, hgflags) && rpmtdCount(&names) > 0) {
-	struct rpmtd_s evr, flags, buildtime; 
-	rpm_time_t * BTp;
+	struct rpmtd_s evr, flags; 
 
 	ds = xcalloc(1, sizeof(*ds));
 	ds->Type = Type;
@@ -162,23 +154,25 @@ rpmds rpmdsNew(Header h, rpmTag tagN, int flags)
 	ds->EVR = evr.data;
 	headerGet(h, tagF, &flags, hgflags);
 	ds->Flags = flags.data;
+	/* ensure rpmlib() requires always have RPMSENSE_RPMLIB flag set */
+	if (tagN == RPMTAG_REQUIRENAME && ds->Flags) {
+	    for (int i = 0; i < ds->Count; i++) {
+		if (!(ds->Flags[i] & RPMSENSE_RPMLIB) &&
+			rstreqn(ds->N[i], "rpmlib(", sizeof("rpmlib(")-1))
+		    ds->Flags[i] |= RPMSENSE_RPMLIB;
+	    }
+	}
 
-	headerGet(h, RPMTAG_BUILDTIME, &buildtime, HEADERGET_MINMEM);
-	BTp = rpmtdGetUint32(&buildtime);
-	ds->BT = BTp ? *BTp : 0;
-	rpmtdFreeData(&buildtime);
+	ds->BT = headerGetNumber(h, RPMTAG_BUILDTIME);
 	ds->Color = xcalloc(ds->Count, sizeof(*ds->Color));
 	ds->Refs = xcalloc(ds->Count, sizeof(*ds->Refs));
+	ds = rpmdsLink(ds, ds->Type);
 
 if (_rpmds_debug < 0)
 fprintf(stderr, "*** ds %p\t%s[%d]\n", ds, ds->Type, ds->Count);
-
     }
 
 exit:
-    /* FIX: ds->Flags may be NULL */
-    ds = rpmdsLink(ds, (ds ? ds->Type : RPMDBG()));
-
     return ds;
 }
 
@@ -228,37 +222,10 @@ char * rpmdsNewDNEVR(const char * dspfx, const rpmds ds)
 
 rpmds rpmdsThis(Header h, rpmTag tagN, rpmsenseFlags Flags)
 {
-    rpmds ds = NULL;
-    const char * Type;
-    const char * n;
-    char *evr;
-
-    if (dsType(tagN, &Type, NULL, NULL))
-	goto exit;
-
-    evr = headerGetEVR(h, &n);
-
-    ds = xcalloc(1, sizeof(*ds));
-    ds->Type = Type;
-    ds->tagN = tagN;
-    ds->Count = 1;
-    ds->nopromote = _rpmds_nopromote;
-    ds->N = rpmdsDupArgv(&n, 1);
-    ds->EVR = rpmdsDupArgv((const char **)&evr, 1);
+    char *evr = headerGetAsString(h, RPMTAG_EVR);
+    rpmds ds = rpmdsSingle(tagN, headerGetString(h, RPMTAG_NAME), evr, Flags);
     free(evr);
-
-    ds->Flags = xmalloc(sizeof(*ds->Flags));	ds->Flags[0] = Flags;
-    ds->i = 0;
-    {	char pre[2];
-	pre[0] = ds->Type[0];
-	pre[1] = '\0';
-	/* LCL: ds->Type may be NULL ??? */
-	ds->DNEVR = rpmdsNewDNEVR(pre, ds);
-    }
-
-   
-exit:
-    return rpmdsLink(ds, (ds ? ds->Type : RPMDBG()));
+    return ds;
 }
 
 rpmds rpmdsSingle(rpmTag tagN, const char * N, const char * EVR, rpmsenseFlags Flags)
@@ -284,11 +251,6 @@ rpmds rpmdsSingle(rpmTag tagN, const char * N, const char * EVR, rpmsenseFlags F
     ds->Flags = xmalloc(sizeof(*ds->Flags));
     ds->Flags[0] = Flags;
     ds->i = 0;
-    {	char t[2];
-	t[0] = ds->Type[0];
-	t[1] = '\0';
-	ds->DNEVR = rpmdsNewDNEVR(t, ds);
-    }
 
 exit:
     return rpmdsLink(ds, (ds ? ds->Type : NULL));
@@ -311,6 +273,7 @@ int rpmdsSetIx(rpmds ds, int ix)
     if (ds != NULL) {
 	i = ds->i;
 	ds->i = ix;
+	ds->DNEVR = _free(ds->DNEVR);
     }
     return i;
 }
@@ -320,8 +283,11 @@ const char * rpmdsDNEVR(const rpmds ds)
     const char * DNEVR = NULL;
 
     if (ds != NULL && ds->i >= 0 && ds->i < ds->Count) {
-	if (ds->DNEVR != NULL)
-	    DNEVR = ds->DNEVR;
+	if (ds->DNEVR == NULL) {
+	    char t[2] = { ds->Type[0], '\0' };
+	    ds->DNEVR = rpmdsNewDNEVR(t, ds);
+	}
+	DNEVR = ds->DNEVR;
     }
     return DNEVR;
 }
@@ -456,13 +422,17 @@ int32_t rpmdsSetRefs(const rpmds ds, int32_t refs)
 
 void rpmdsNotify(rpmds ds, const char * where, int rc)
 {
+    const char *DNEVR;
+
+    if (!rpmIsDebug())
+	return;
     if (!(ds != NULL && ds->i >= 0 && ds->i < ds->Count))
 	return;
-    if (!(ds->Type != NULL && ds->DNEVR != NULL))
+    if (!(ds->Type != NULL && (DNEVR = rpmdsDNEVR(ds)) != NULL))
 	return;
 
     rpmlog(RPMLOG_DEBUG, "%9s: %-45s %-s %s\n", ds->Type,
-		(!strcmp(ds->DNEVR, "cached") ? ds->DNEVR : ds->DNEVR+2),
+		(rstreq(DNEVR, "cached") ? DNEVR : DNEVR+2),
 		(rc ? _("NO ") : _("YES")),
 		(where != NULL ? where : ""));
 }
@@ -473,13 +443,8 @@ int rpmdsNext(rpmds ds)
 
     if (ds != NULL && ++ds->i >= 0) {
 	if (ds->i < ds->Count) {
-	    char t[2];
 	    i = ds->i;
 	    ds->DNEVR = _free(ds->DNEVR);
-	    t[0] = ((ds->Type != NULL) ? ds->Type[0] : '\0');
-	    t[1] = '\0';
-	    ds->DNEVR = rpmdsNewDNEVR(t, ds);
-
 	} else
 	    ds->i = -1;
 
@@ -493,8 +458,10 @@ fprintf(stderr, "*** ds %p\t%s[%d]: %s\n", ds, (ds->Type ? ds->Type : "?Type?"),
 
 rpmds rpmdsInit(rpmds ds)
 {
-    if (ds != NULL)
+    if (ds != NULL) {
 	ds->i = -1;
+	ds->DNEVR = _free(ds->DNEVR);
+    }
     return ds;
 }
 
@@ -536,14 +503,12 @@ static rpmds rpmdsDup(const rpmds ods)
     ds->u = ods->u;
     ds->nopromote = ods->nopromote;
 
-    nb = (ds->Count+1) * sizeof(*ds->N);
     ds->N = rpmdsDupArgv(ods->N, ods->Count);
 
     /* XXX rpm prior to 3.0.2 did not always supply EVR and Flags. */
 assert(ods->EVR != NULL);
 assert(ods->Flags != NULL);
 
-    nb = (ds->Count+1) * sizeof(*ds->EVR);
     ds->EVR = rpmdsDupArgv(ods->EVR, ods->Count);
 
     nb = (ds->Count * sizeof(*ds->Flags));
@@ -612,7 +577,6 @@ int rpmdsMerge(rpmds * dsp, rpmds ods)
      */
     save = ods->i;
     ods = rpmdsInit(ods);
-    if (ods != NULL)
     while (rpmdsNext(ods) >= 0) {
 	/*
 	 * If this entry is already present, don't bother.
@@ -683,15 +647,15 @@ int rpmdsSearch(rpmds ds, rpmds ods)
 	    l = i + 1;
 	else {
 	    /* Set l to 1st member of set that contains N. */
-	    if (strcmp(ods->N[ods->i], ds->N[l]))
+	    if (!rstreq(ods->N[ods->i], ds->N[l]))
 		l = i;
-	    while (l > 0 && !strcmp(ods->N[ods->i], ds->N[l-1]))
+	    while (l > 0 && rstreq(ods->N[ods->i], ds->N[l-1]))
 		l--;
 	    /* Set u to 1st member of set that does not contain N. */
-	    if (u >= ds->Count || strcmp(ods->N[ods->i], ds->N[u]))
+	    if (u >= ds->Count || !rstreq(ods->N[ods->i], ds->N[u]))
 		u = i;
 	    while (++u < ds->Count) {
-		if (strcmp(ods->N[ods->i], ds->N[u]))
+		if (!rstreq(ods->N[ods->i], ds->N[u]))
 		    /*@innerbreak@*/ break;
 	    }
 	    break;
@@ -761,15 +725,13 @@ void parseEVR(char * evr,
 
 int rpmdsCompare(const rpmds A, const rpmds B)
 {
-    const char *aDepend = (A->DNEVR != NULL ? A->DNEVR+2 : "");
-    const char *bDepend = (B->DNEVR != NULL ? B->DNEVR+2 : "");
     char *aEVR, *bEVR;
     const char *aE, *aV, *aR, *bE, *bV, *bR;
     int result;
     int sense;
 
     /* Different names don't overlap. */
-    if (strcmp(A->N[A->i], B->N[B->i])) {
+    if (!rstreq(A->N[A->i], B->N[B->i])) {
 	result = 0;
 	goto exit;
     }
@@ -804,9 +766,6 @@ int rpmdsCompare(const rpmds A, const rpmds B)
 	sense = rpmvercmp(aE, bE);
     else if (aE && *aE && atol(aE) > 0) {
 	if (!B->nopromote) {
-	    int lvl = (_rpmds_unspecified_epoch_noise  ? RPMLOG_WARNING : RPMLOG_DEBUG);
-	    rpmlog(lvl, _("The \"B\" dependency needs an epoch (assuming same epoch as \"A\")\n\tA = \"%s\"\tB = \"%s\"\n"),
-		aDepend, bDepend);
 	    sense = 0;
 	} else
 	    sense = 1;
@@ -835,16 +794,12 @@ int rpmdsCompare(const rpmds A, const rpmds B)
     }
 
 exit:
-    if (_noisy_range_comparison_debug_message)
-    rpmlog(RPMLOG_DEBUG, "  %s    A %s\tB %s\n",
-	(result ? "YES" : "NO "), aDepend, bDepend);
     return result;
 }
 
 void rpmdsProblem(rpmps ps, const char * pkgNEVR, const rpmds ds,
 	const fnpyKey * suggestedKeys, int adding)
 {
-    const char * Name =  rpmdsN(ds);
     const char * DNEVR = rpmdsDNEVR(ds);
     const char * EVR = rpmdsEVR(ds);
     rpmProblemType type;
@@ -852,7 +807,6 @@ void rpmdsProblem(rpmps ps, const char * pkgNEVR, const rpmds ds,
 
     if (ps == NULL) return;
 
-    if (Name == NULL) Name = "?N?";
     if (EVR == NULL) EVR = "?EVR?";
     if (DNEVR == NULL) DNEVR = "? ?N? ?OP? ?EVR?";
 
@@ -903,7 +857,7 @@ int rpmdsAnyMatchesDep (const Header h, const rpmds req, int nopromote)
     while (rpmdsNext(provides) >= 0) {
 
 	/* Filter out provides that came along for the ride. */
-	if (strcmp(provides->N[provides->i], req->N[req->i]))
+	if (!rstreq(provides->N[provides->i], req->N[req->i]))
 	    continue;
 
 	result = rpmdsCompare(provides, req);
@@ -935,7 +889,8 @@ int rpmdsNVRMatchesDep(const Header h, const rpmds req, int nopromote)
 	return rc;
 
     /* Get package information from header */
-    pkgEVR = headerGetEVR(h, &pkgN);
+    pkgN = headerGetString(h, RPMTAG_NAME);
+    pkgEVR = headerGetAsString(h, RPMTAG_EVR);
     if ((pkg = rpmdsSingle(RPMTAG_PROVIDENAME, pkgN, pkgEVR, pkgFlags)) != NULL) {
 	if (nopromote)
 	    (void) rpmdsSetNoPromote(pkg, nopromote);
