@@ -246,7 +246,7 @@ typedef enum rpmTag_e {
     RPMTAG_FLINKHDRID		= 1168, /* s[] (unimplemented) */
     RPMTAG_FLINKNEVRA		= 1169, /* s[] (unimplemented) */
     RPMTAG_PACKAGEORIGIN	= 1170, /* s (unimplemented) */
-    RPMTAG_TRIGGERPREIN		= 1171, /* internal (unimplemented) */
+    RPMTAG_TRIGGERPREIN		= 1171, /* internal */
     RPMTAG_BUILDSUGGESTS	= 1172, /* internal (unimplemented) */
     RPMTAG_BUILDENHANCES	= 1173, /* internal (unimplemented) */
     RPMTAG_SCRIPTSTATES		= 1174, /* i[] scriptlet exit codes (unimplemented) */
@@ -270,6 +270,8 @@ typedef enum rpmTag_e {
     RPMTAG_REQUIREATTRSX	= 1192, /* i[] (unimplemented) */
     RPMTAG_BUILDPROVIDES	= 1193, /* internal */
     RPMTAG_BUILDOBSOLETES	= 1194, /* internal */
+    RPMTAG_DBINSTANCE		= 1195, /* i extension */
+    RPMTAG_NVRA			= 1196, /* s extension */
     RPMTAG_FILENAMES		= 5000, /* s[] extension */
     RPMTAG_FILEPROVIDE		= 5001, /* s[] extension */
     RPMTAG_FILEREQUIRE		= 5002, /* s[] extension */
@@ -282,6 +284,14 @@ typedef enum rpmTag_e {
     RPMTAG_LONGSIZE		= 5009, /* l */
     RPMTAG_FILECAPS		= 5010, /* s[] */
     RPMTAG_FILEDIGESTALGO	= 5011, /* i file digest algorithm */
+    RPMTAG_BUGURL		= 5012, /* s */
+    RPMTAG_EVR			= 5013, /* s extension */
+    RPMTAG_NVR			= 5014, /* s extension */
+    RPMTAG_NEVR			= 5015, /* s extension */
+    RPMTAG_NEVRA		= 5016, /* s extension */
+    RPMTAG_HEADERCOLOR		= 5017, /* i extension */
+    RPMTAG_VERBOSE		= 5018, /* i extension */
+    RPMTAG_EPOCHNUM		= 5019, /* i extension */
 
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
 } rpmTag;
