@@ -15,36 +15,17 @@
 #include <rpm/rpmkeyring.h>
 
 #include <rpm/rpmdb.h>
-#include <rpm/rpmal.h>
 #include <rpm/rpmds.h>
 #include <rpm/rpmfi.h>
 #include <rpm/rpmlog.h>
 #include <rpm/rpmte.h>
 
 #include "rpmio/digest.h"
+#include "lib/rpmal.h"
 #include "lib/rpmlock.h"
 #include "lib/rpmts_internal.h"
 #include "lib/misc.h"
 
-/* XXX FIXME: merge with existing (broken?) tests in system.h */
-/* portability fiddles */
-#if STATFS_IN_SYS_STATVFS
-#include <sys/statvfs.h>
-
-#else
-# if STATFS_IN_SYS_VFS
-#  include <sys/vfs.h>
-# else
-#  if STATFS_IN_SYS_MOUNT
-#   include <sys/mount.h>
-#  else
-#   if STATFS_IN_SYS_STATFS
-#    include <sys/statfs.h>
-#   endif
-#  endif
-# endif
-#endif
-
 #include "debug.h"
 
 static void loadKeyring(rpmts ts);
@@ -249,9 +230,9 @@ rpmKeyring rpmtsGetKeyring(rpmts ts, int autoload)
 	if (ts->keyring == NULL && autoload) {
 	    loadKeyring(ts);
 	}
-	keyring = ts->keyring;
+	keyring = rpmKeyringLink(ts->keyring);
     }
-    return rpmKeyringLink(keyring);
+    return keyring;
 }
 
 int rpmtsSetKeyring(rpmts ts, rpmKeyring keyring)
@@ -321,7 +302,7 @@ static int loadKeyringFromDB(rpmts ts)
 	    if (b64decode(key, (void **) &pkt, &pktlen) == 0) {
 		rpmPubkey key = rpmPubkeyNew(pkt, pktlen);
 		if (rpmKeyringAddKey(ts->keyring, key) == 0) {
-		    char *nvr = headerGetNEVR(h, NULL);
+		    char *nvr = headerGetAsString(h, RPMTAG_NVR);
 		    rpmlog(RPMLOG_DEBUG, "added key %s to keyring\n", nvr);
 		    free(nvr);
 		    nkeys++;
@@ -631,17 +612,6 @@ rpmVSFlags rpmtsSetVSFlags(rpmts ts, rpmVSFlags vsflags)
     return ovsflags;
 }
 
-int rpmtsUnorderedSuccessors(rpmts ts, int first)
-{
-    int unorderedSuccessors = 0;
-    if (ts != NULL) {
-	unorderedSuccessors = ts->unorderedSuccessors;
-	if (first >= 0)
-	    ts->unorderedSuccessors = first;
-    }
-    return unorderedSuccessors;
-}
-
 const char * rpmtsRootDir(rpmts ts)
 {
     const char * rootDir = NULL;
@@ -675,7 +645,7 @@ int rpmtsSetRootDir(rpmts ts, const char * rootDir)
     ts->rootDir = _free(ts->rootDir);
     /* Ensure clean path with a trailing slash */
     ts->rootDir = rootDir ? rpmGetPath(rootDir, NULL) : xstrdup("/");
-    if (strcmp(ts->rootDir, "/") != 0) {
+    if (!rstreq(ts->rootDir, "/")) {
 	rstrcat(&ts->rootDir, "/");
     }
     return 0;
@@ -772,171 +742,6 @@ rpmdb rpmtsGetRdb(rpmts ts)
     return rdb;
 }
 
-int rpmtsInitDSI(const rpmts ts)
-{
-    rpmDiskSpaceInfo dsi;
-    struct stat sb;
-    int rc;
-    int i;
-
-    if (rpmtsFilterFlags(ts) & RPMPROB_FILTER_DISKSPACE)
-	return 0;
-
-    rpmlog(RPMLOG_DEBUG, "mounted filesystems:\n");
-    rpmlog(RPMLOG_DEBUG,
-	"    i        dev    bsize       bavail       iavail mount point\n");
-
-    rc = rpmGetFilesystemList(&ts->filesystems, &ts->filesystemCount);
-    if (rc || ts->filesystems == NULL || ts->filesystemCount <= 0)
-	return rc;
-
-    /* Get available space on mounted file systems. */
-
-    ts->dsi = _free(ts->dsi);
-    ts->dsi = xcalloc((ts->filesystemCount + 1), sizeof(*ts->dsi));
-
-    dsi = ts->dsi;
-
-    if (dsi != NULL)
-    for (i = 0; (i < ts->filesystemCount) && dsi; i++, dsi++) {
-#if STATFS_IN_SYS_STATVFS
-	struct statvfs sfb;
-	memset(&sfb, 0, sizeof(sfb));
-	rc = statvfs(ts->filesystems[i], &sfb);
-#else
-	struct statfs sfb;
-	memset(&sfb, 0, sizeof(sfb));
-#  if STAT_STATFS4
-/* This platform has the 4-argument version of the statfs call.  The last two
- * should be the size of struct statfs and 0, respectively.  The 0 is the
- * filesystem type, and is always 0 when statfs is called on a mounted
- * filesystem, as we're doing.
- */
-	rc = statfs(ts->filesystems[i], &sfb, sizeof(sfb), 0);
-#  else
-	rc = statfs(ts->filesystems[i], &sfb);
-#  endif
-#endif
-	if (rc)
-	    break;
-
-	rc = stat(ts->filesystems[i], &sb);
-	if (rc)
-	    break;
-	dsi->dev = sb.st_dev;
-
-	dsi->bsize = sfb.f_bsize;
-	dsi->bneeded = 0;
-	dsi->ineeded = 0;
-#ifdef STATFS_HAS_F_BAVAIL
-	dsi->bavail = sfb.f_bavail;
-#else
-/* FIXME: the statfs struct doesn't have a member to tell how many blocks are
- * available for non-superusers.  f_blocks - f_bfree is probably too big, but
- * it's about all we can do.
- */
-	dsi->bavail = sfb.f_blocks - sfb.f_bfree;
-#endif
-	/* XXX Avoid FAT and other file systems that have not inodes. */
-	/* XXX assigning negative value to unsigned type */
-	dsi->iavail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
-				? sfb.f_ffree : -1;
-	rpmlog(RPMLOG_DEBUG, 
-		"%5d 0x%08x %8" PRId64 " %12" PRId64 " %12" PRId64" %s\n",
-		i, (unsigned) dsi->dev, dsi->bsize,
-		dsi->bavail, dsi->iavail,
-		ts->filesystems[i]);
-    }
-    return rc;
-}
-
-void rpmtsUpdateDSI(const rpmts ts, dev_t dev,
-		rpm_loff_t fileSize, rpm_loff_t prevSize, rpm_loff_t fixupSize,
-		rpmFileAction action)
-{
-    rpmDiskSpaceInfo dsi;
-    int64_t bneeded;
-
-    dsi = ts->dsi;
-    if (dsi) {
-	while (dsi->bsize && dsi->dev != dev)
-	    dsi++;
-	if (dsi->bsize == 0)
-	    dsi = NULL;
-    }
-    if (dsi == NULL)
-	return;
-
-    bneeded = BLOCK_ROUND(fileSize, dsi->bsize);
-
-    switch (action) {
-    case FA_BACKUP:
-    case FA_SAVE:
-    case FA_ALTNAME:
-	dsi->ineeded++;
-	dsi->bneeded += bneeded;
-	break;
-
-    /*
-     * FIXME: If two packages share a file (same md5sum), and
-     * that file is being replaced on disk, will dsi->bneeded get
-     * adjusted twice? Quite probably!
-     */
-    case FA_CREATE:
-	dsi->bneeded += bneeded;
-	dsi->bneeded -= BLOCK_ROUND(prevSize, dsi->bsize);
-	break;
-
-    case FA_ERASE:
-	dsi->ineeded--;
-	dsi->bneeded -= bneeded;
-	break;
-
-    default:
-	break;
-    }
-
-    if (fixupSize)
-	dsi->bneeded -= BLOCK_ROUND(fixupSize, dsi->bsize);
-}
-
-void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
-{
-    rpmDiskSpaceInfo dsi;
-    rpmps ps;
-    int fc;
-    int i;
-
-    if (ts->filesystems == NULL || ts->filesystemCount <= 0)
-	return;
-
-    dsi = ts->dsi;
-    if (dsi == NULL)
-	return;
-    fc = rpmfiFC(rpmteFI(te));
-    if (fc <= 0)
-	return;
-
-    ps = rpmtsProblems(ts);
-    for (i = 0; i < ts->filesystemCount; i++, dsi++) {
-
-	if (dsi->bavail >= 0 && adj_fs_blocks(dsi->bneeded) > dsi->bavail) {
-	    rpmpsAppend(ps, RPMPROB_DISKSPACE,
-			rpmteNEVRA(te), rpmteKey(te),
-			ts->filesystems[i], NULL, NULL,
- 	   (adj_fs_blocks(dsi->bneeded) - dsi->bavail) * dsi->bsize);
-	}
-
-	if (dsi->iavail >= 0 && adj_fs_blocks(dsi->ineeded) > dsi->iavail) {
-	    rpmpsAppend(ps, RPMPROB_DISKNODES,
-			rpmteNEVRA(te), rpmteKey(te),
-			ts->filesystems[i], NULL, NULL,
- 	    (adj_fs_blocks(dsi->ineeded) - dsi->iavail));
-	}
-    }
-    ps = rpmpsFree(ps);
-}
-
 void * rpmtsNotify(rpmts ts, rpmte te,
 		rpmCallbackType what, rpm_loff_t amount, rpm_loff_t total)
 {
@@ -1028,6 +833,16 @@ rpm_color_t rpmtsPrefColor(rpmts ts)
     return (ts != NULL ? ts->prefcolor : 0);
 }
 
+rpm_color_t rpmtsSetPrefColor(rpmts ts, rpm_color_t color)
+{
+    rpm_color_t ocolor = 0;
+    if (ts != NULL) {
+	ocolor = ts->prefcolor;
+	ts->prefcolor = color;
+    }
+    return ocolor;
+}
+
 rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
 {
     rpmop op = NULL;
@@ -1047,34 +862,6 @@ int rpmtsSetNotifyCallback(rpmts ts,
     return 0;
 }
 
-int rpmtsGetKeys(const rpmts ts, fnpyKey ** ep, int * nep)
-{
-    int rc = 0;
-
-    if (nep) *nep = ts->orderCount;
-    if (ep) {
-	rpmtsi pi;	rpmte p;
-	fnpyKey * e;
-
-	*ep = e = xmalloc(ts->orderCount * sizeof(*e));
-	pi = rpmtsiInit(ts);
-	while ((p = rpmtsiNext(pi, 0)) != NULL) {
-	    switch (rpmteType(p)) {
-	    case TR_ADDED:
-		*e = rpmteKey(p);
-		break;
-	    case TR_REMOVED:
-	    default:
-		*e = NULL;
-		break;
-	    }
-	    e++;
-	}
-	pi = rpmtsiFree(pi);
-    }
-    return rc;
-}
-
 rpmts rpmtsCreate(void)
 {
     rpmts ts;
@@ -1113,7 +900,7 @@ rpmts rpmtsCreate(void)
 	    argvSplit(&langs, tmp, ":");	
 	    /* If we'll be installing all languages anyway, don't bother */
 	    for (ARGV_t l = langs; *l; l++) {
-		if (strcmp(*l, "all") == 0) {
+		if (rstreq(*l, "all")) {
 		    langs = argvFree(langs);
 		    break;
 		}
@@ -1149,9 +936,6 @@ rpmts rpmtsCreate(void)
 
     ts->nrefs = 0;
 
-    /* make sure crypto gets initialized before we might go chrooting */
-    rpmInitCrypto();
-
     return rpmtsLink(ts, RPMDBG_M("tsCreate"));
 }
 
