@@ -9,29 +9,149 @@
 #define	_USE_LIBIO	1
 #endif
 
-/* XXX HP-UX w/o -D_XOPEN_SOURCE needs */
-#if !defined(HAVE_HERRNO) && (defined(__hpux))
-extern int h_errno;
-#endif
-
-#ifndef IPPORT_FTP
-#define IPPORT_FTP	21
-#endif
-#ifndef	IPPORT_HTTP
-#define	IPPORT_HTTP	80
-#endif
-
 #include <rpm/rpmlog.h>
 #include <rpm/rpmmacro.h>
 #include <rpm/rpmfileutil.h>
+#include <rpm/rpmsw.h>
+#include <rpm/rpmurl.h>
 
 #include "rpmio/rpmio_internal.h"
 
 #include "debug.h"
 
+typedef struct _FDSTACK_s {
+    FDIO_t		io;
+    void *		fp;
+    int			fdno;
+} FDSTACK_t;
+
+/** \ingroup rpmio
+ * Cumulative statistics for a descriptor.
+ */
+typedef	struct {
+    struct rpmop_s	ops[FDSTAT_MAX];	/*!< Cumulative statistics. */
+} * FDSTAT_t;
+
+/** \ingroup rpmio
+ * The FD_t File Handle data structure.
+ */
+struct _FD_s {
+    int		nrefs;
+    int		flags;
+#define	RPMIO_DEBUG_IO		0x40000000
+#define	RPMIO_DEBUG_REFS	0x20000000
+    int		magic;
+#define	FDMAGIC			0x04463138
+    int		nfps;
+    FDSTACK_t	fps[8];
+    int		urlType;	/* ufdio: */
+
+    ssize_t	bytesRemain;	/* ufdio: */
+
+    int		syserrno;	/* last system errno encountered */
+    const void *errcookie;	/* gzdio/bzdio/ufdio/xzdio: */
+
+    FDSTAT_t	stats;		/* I/O statistics */
+
+    rpmDigestBundle digests;
+};
+
+#define	FDSANE(fd)	assert(fd && fd->magic == FDMAGIC)
+
+#define DBG(_f, _m, _x) \
+    \
+    if ((_rpmio_debug | ((_f) ? ((FD_t)(_f))->flags : 0)) & (_m)) fprintf _x \
+
+#define DBGIO(_f, _x)   DBG((_f), RPMIO_DEBUG_IO, _x)
+#define DBGREFS(_f, _x) DBG((_f), RPMIO_DEBUG_REFS, _x)
+
+static FDIO_t fdGetIo(FD_t fd)
+{
+    FDSANE(fd);
+    return fd->fps[fd->nfps].io;
+}
+
+static void fdSetIo(FD_t fd, FDIO_t io)
+{
+    FDSANE(fd);
+    fd->fps[fd->nfps].io = io;
+}
+
+static void * fdGetFp(FD_t fd)
+{
+    FDSANE(fd);
+    return fd->fps[fd->nfps].fp;
+}
+
+static void fdSetFp(FD_t fd, void * fp)
+{
+    FDSANE(fd);
+    fd->fps[fd->nfps].fp = fp;
+}
+
+static void fdSetFdno(FD_t fd, int fdno)
+{
+    FDSANE(fd);
+    fd->fps[fd->nfps].fdno = fdno;
+}
+
+static void fdPush(FD_t fd, FDIO_t io, void * fp, int fdno)
+{
+    FDSANE(fd);
+    if (fd->nfps >= (sizeof(fd->fps)/sizeof(fd->fps[0]) - 1))
+	return;
+    fd->nfps++;
+    fdSetIo(fd, io);
+    fdSetFp(fd, fp);
+    fdSetFdno(fd, fdno);
+}
+
+static void fdPop(FD_t fd)
+{
+    FDSANE(fd);
+    if (fd->nfps < 0) return;
+    fdSetIo(fd, NULL);
+    fdSetFp(fd, NULL);
+    fdSetFdno(fd, -1);
+    fd->nfps--;
+}
+
+static FD_t c2f(void * cookie)
+{
+    FD_t fd = (FD_t) cookie;
+    FDSANE(fd);
+    return fd;
+}
+
+void fdSetBundle(FD_t fd, rpmDigestBundle bundle)
+{
+    FDSANE(fd);
+    fd->digests = bundle;
+}
+
+rpmDigestBundle fdGetBundle(FD_t fd)
+{
+    FDSANE(fd);
+    return fd->digests;
+}
+
+static void * iotFileno(FD_t fd, FDIO_t iot)
+{
+    void * rc = NULL;
+
+    FDSANE(fd);
+    for (int i = fd->nfps; i >= 0; i--) {
+	FDSTACK_t * fps = &fd->fps[i];
+	if (fps->io != iot)
+	    continue;
+	rc = fps->fp;
+	break;
+    }
+    
+    return rc;
+}
+
 #define FDNREFS(fd)	(fd ? ((FD_t)fd)->nrefs : -9)
-#define FDTO(fd)	(fd ? ((FD_t)fd)->rd_timeoutsecs : -99)
-#define FDCPIOPOS(fd)	(fd ? ((FD_t)fd)->fd_cpioPos : -99)
 
 #define	FDONLY(fd)	assert(fdGetIo(fd) == fdio)
 #define	GZDONLY(fd)	assert(fdGetIo(fd) == gzdio)
@@ -48,8 +168,6 @@ static int noLibio = 0;
 static int noLibio = 1;
 #endif
 
-#define TIMEOUT_SECS 60
-
 /** \ingroup rpmio
  * \name RPMIO Vectors.
  */
@@ -94,6 +212,11 @@ static const FDIO_t bzdio;
 static const FDIO_t xzdio;
 static const FDIO_t lzdio;
 
+/** \ingroup rpmio
+ * Update digest(s) attached to fd.
+ */
+static void fdUpdateDigests(FD_t fd, const unsigned char * buf, size_t buflen);
+
 /**
  */
 int _rpmio_debug = 0;
@@ -110,13 +233,6 @@ static const char * fdbg(FD_t fd)
     if (fd == NULL)
 	return buf;
 
-#ifdef DYING
-    sprintf(be, "fd %p", fd);	be += strlen(be);
-    if (fd->rd_timeoutsecs >= 0) {
-	sprintf(be, " secs %d", fd->rd_timeoutsecs);
-	be += strlen(be);
-    }
-#endif
     if (fd->bytesRemain != -1) {
 	sprintf(be, " clen %d", (int)fd->bytesRemain);
 	be += strlen(be);
@@ -160,28 +276,70 @@ static const char * fdbg(FD_t fd)
     return buf;
 }
 
+static void fdstat_enter(FD_t fd, int opx)
+{
+    if (fd == NULL) return;
+    if (fd->stats != NULL)
+	(void) rpmswEnter(fdOp(fd, opx), (ssize_t) 0);
+}
+
+static void fdstat_exit(FD_t fd, int opx, ssize_t rc)
+{
+    if (fd == NULL) return;
+    if (rc == -1)
+	fd->syserrno = errno;
+    else if (rc > 0 && fd->bytesRemain > 0)
+	switch (opx) {
+	case FDSTAT_READ:
+	case FDSTAT_WRITE:
+	fd->bytesRemain -= rc;
+	    break;
+	default:
+	    break;
+	}
+    if (fd->stats != NULL)
+	(void) rpmswExit(fdOp(fd, opx), rc);
+}
+
+static void fdstat_print(FD_t fd, const char * msg, FILE * fp)
+{
+    static const int usec_scale = (1000*1000);
+    int opx;
+
+    if (fd == NULL || fd->stats == NULL) return;
+    for (opx = 0; opx < 4; opx++) {
+	rpmop op = &fd->stats->ops[opx];
+	if (op->count <= 0) continue;
+	switch (opx) {
+	case FDSTAT_READ:
+	    if (msg) fprintf(fp, "%s:", msg);
+	    fprintf(fp, "%8d reads, %8ld total bytes in %d.%06d secs\n",
+		op->count, (long)op->bytes,
+		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
+	    break;
+	case FDSTAT_WRITE:
+	    if (msg) fprintf(fp, "%s:", msg);
+	    fprintf(fp, "%8d writes, %8ld total bytes in %d.%06d secs\n",
+		op->count, (long)op->bytes,
+		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
+	    break;
+	case FDSTAT_SEEK:
+	    break;
+	case FDSTAT_CLOSE:
+	    break;
+	}
+    }
+}
+
 /* =============================================================== */
 off_t fdSize(FD_t fd)
 {
     struct stat sb;
     off_t rc = -1; 
 
-#ifdef	NOISY
-DBGIO(0, (stderr, "==>\tfdSize(%p) rc %ld\n", fd, (long)rc));
-#endif
     FDSANE(fd);
-    switch (fd->urlType) {
-    case URL_IS_PATH:
-    case URL_IS_UNKNOWN:
-	if (fstat(Fileno(fd), &sb) == 0)
-	    rc = sb.st_size;
-    case URL_IS_HTTPS:
-    case URL_IS_HTTP:
-    case URL_IS_HKP:
-    case URL_IS_FTP:
-    case URL_IS_DASH:
-	break;
-    }
+    if (fstat(Fileno(fd), &sb) == 0)
+	rc = sb.st_size;
     return rc;
 }
 
@@ -198,8 +356,7 @@ DBGIO(fd, (stderr, "==> fdDup(%d) fd %p %s\n", fdno, (fd ? fd : NULL), fdbg(fd))
     return fd;
 }
 
-static inline int fdSeekNot(void * cookie,
-		_libio_pos_t pos,  int whence)
+static int fdSeekNot(void * cookie, _libio_pos_t pos,  int whence)
 {
     FD_t fd = c2f(cookie);
     FDSANE(fd);		/* XXX keep gcc quiet */
@@ -208,8 +365,7 @@ static inline int fdSeekNot(void * cookie,
 
 /** \ingroup rpmio
  */
-static inline
-int fdFileno(void * cookie)
+static int fdFileno(void * cookie)
 {
     FD_t fd;
     if (cookie == NULL) return -2;
@@ -241,8 +397,6 @@ DBGREFS(fd, (stderr, "--> fd  %p ++ %d %s  %s\n", fd, fd->nrefs, msg, fdbg(fd)))
  */
 FD_t fdFree( FD_t fd, const char *msg)
 {
-	int i;
-
 if (fd == NULL)
 DBGREFS(0, (stderr, "--> fd  %p -- %d %s\n", fd, FDNREFS(fd), msg));
     FDSANE(fd);
@@ -251,14 +405,9 @@ DBGREFS(fd, (stderr, "--> fd  %p -- %d %s %s\n", fd, fd->nrefs, msg, fdbg(fd)));
 	if (--fd->nrefs > 0)
 	    return fd;
 	fd->stats = _free(fd->stats);
-	for (i = fd->ndigests - 1; i >= 0; i--) {
-	    FDDIGEST_t fddig = fd->digests + i;
-	    if (fddig->hashctx == NULL)
-		continue;
-	    (void) rpmDigestFinal(fddig->hashctx, NULL, NULL, 0);
-	    fddig->hashctx = NULL;
+	if (fd->digests) {
+	    fd->digests = rpmDigestBundleFree(fd->digests);
 	}
-	fd->ndigests = 0;
 	free(fd);
     }
     return NULL;
@@ -283,16 +432,11 @@ FD_t fdNew(const char * msg)
     fd->fps[0].fp = NULL;
     fd->fps[0].fdno = -1;
 
-    fd->rd_timeoutsecs = 1;	/* XXX default value used to be -1 */
     fd->bytesRemain = -1;
     fd->syserrno = 0;
     fd->errcookie = NULL;
     fd->stats = xcalloc(1, sizeof(*fd->stats));
-
-    fd->ndigests = 0;
-    memset(fd->digests, 0, sizeof(fd->digests));
-
-    fd->fd_cpioPos = 0;
+    fd->digests = NULL;
 
     return fdLink(fd, msg);
 }
@@ -310,7 +454,7 @@ static ssize_t fdRead(void * cookie, char * buf, size_t count)
     rc = read(fdFileno(fd), buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
     fdstat_exit(fd, FDSTAT_READ, rc);
 
-    if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
+    if (fd->digests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
 
 DBGIO(fd, (stderr, "==>\tfdRead(%p,%p,%ld) rc %ld %s\n", cookie, buf, (long)count, (long)rc, fdbg(fd)));
 
@@ -327,7 +471,7 @@ static ssize_t fdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd->bytesRemain == 0) return 0;	/* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
+    if (fd->digests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     if (count == 0) return 0;
 
@@ -340,7 +484,7 @@ DBGIO(fd, (stderr, "==>\tfdWrite(%p,%p,%ld) rc %ld %s\n", cookie, buf, (long)cou
     return rc;
 }
 
-static inline int fdSeek(void * cookie, _libio_pos_t pos, int whence)
+static int fdSeek(void * cookie, _libio_pos_t pos, int whence)
 {
 #ifdef USE_COOKIE_SEEK_POINTER
     _IO_off64_t p = *pos;
@@ -441,7 +585,6 @@ int fdWritable(FD_t fd, int secs)
 	rc = select(fdno + 1, NULL, &wrfds, NULL, tvp);
 #endif
 
-	/* HACK: EBADF on PUT chunked termination from ufdClose. */
 if (_rpmio_debug && !(rc == 1 && errno == 0))
 fprintf(stderr, "*** fdWritable fdno %d rc %d %s\n", fdno, rc, strerror(errno));
 	if (rc < 0) {
@@ -535,160 +678,6 @@ int ufdCopy(FD_t sfd, FD_t tfd)
     return rc;
 }
 
-/* =============================================================== */
-static ssize_t ufdRead(void * cookie, char * buf, size_t count)
-{
-    FD_t fd = c2f(cookie);
-    int bytesRead;
-    int total;
-
-    /* XXX preserve timedRead() behavior */
-    if (fdGetIo(fd) == fdio) {
-	struct stat sb;
-	int fdno = fdFileno(fd);
-	(void) fstat(fdno, &sb);
-	if (S_ISREG(sb.st_mode))
-	    return fdRead(fd, buf, count);
-    }
-
-    UFDONLY(fd);
-    assert(fd->rd_timeoutsecs >= 0);
-
-    for (total = 0; total < count; total += bytesRead) {
-
-	int rc;
-
-	bytesRead = 0;
-
-	/* Is there data to read? */
-	if (fd->bytesRemain == 0) return total;	/* XXX simulate EOF */
-	rc = fdReadable(fd, fd->rd_timeoutsecs);
-
-	switch (rc) {
-	case -1:	/* error */
-	case  0:	/* timeout */
-	    return total;
-	    break;
-	default:	/* data to read */
-	    break;
-	}
-
-	rc = fdRead(fd, buf + total, count - total);
-
-	if (rc < 0) {
-	    switch (errno) {
-	    case EWOULDBLOCK:
-		continue;
-		break;
-	    default:
-		break;
-	    }
-if (_rpmio_debug)
-fprintf(stderr, "*** read: rc %d errno %d %s \"%s\"\n", rc, errno, strerror(errno), buf);
-	    return rc;
-	    break;
-	} else if (rc == 0) {
-	    return total;
-	    break;
-	}
-	bytesRead = rc;
-    }
-
-    return count;
-}
-
-static ssize_t ufdWrite(void * cookie, const char * buf, size_t count)
-{
-    FD_t fd = c2f(cookie);
-    int bytesWritten;
-    int total = 0;
-
-#ifdef	NOTYET
-    if (fdGetIo(fd) == fdio) {
-	struct stat sb;
-	(void) fstat(fdGetFdno(fd), &sb);
-	if (S_ISREG(sb.st_mode))
-	    return fdWrite(fd, buf, count);
-    }
-#endif
-
-    UFDONLY(fd);
-
-    for (total = 0; total < count; total += bytesWritten) {
-
-	int rc;
-
-	bytesWritten = 0;
-
-	/* Is there room to write data? */
-	if (fd->bytesRemain == 0) {
-fprintf(stderr, "*** ufdWrite fd %p WRITE PAST END OF CONTENT\n", fd);
-	    return total;	/* XXX simulate EOF */
-	}
-	rc = fdWritable(fd, 2);		/* XXX configurable? */
-
-	switch (rc) {
-	case -1:	/* error */
-	case  0:	/* timeout */
-	    return total;
-	    break;
-	default:	/* data to write */
-	    break;
-	}
-
-	rc = fdWrite(fd, buf + total, count - total);
-
-	if (rc < 0) {
-	    switch (errno) {
-	    case EWOULDBLOCK:
-		continue;
-		break;
-	    default:
-		break;
-	    }
-if (_rpmio_debug)
-fprintf(stderr, "*** write: rc %d errno %d %s \"%s\"\n", rc, errno, strerror(errno), buf);
-	    return rc;
-	    break;
-	} else if (rc == 0) {
-	    return total;
-	    break;
-	}
-	bytesWritten = rc;
-    }
-
-    return count;
-}
-
-static inline int ufdSeek(void * cookie, _libio_pos_t pos, int whence)
-{
-    FD_t fd = c2f(cookie);
-
-    switch (fd->urlType) {
-    case URL_IS_UNKNOWN:
-    case URL_IS_PATH:
-	break;
-    case URL_IS_HTTPS:
-    case URL_IS_HTTP:
-    case URL_IS_HKP:
-    case URL_IS_FTP:
-    case URL_IS_DASH:
-    default:
-        return -2;
-	break;
-    }
-    return fdSeek(cookie, pos, whence);
-}
-
-static int ufdClose( void * cookie)
-{
-    FD_t fd = c2f(cookie);
-
-    UFDONLY(fd);
-
-    return fdClose(fd);
-}
-
 /*
  * Deal with remote url's by fetching them with a helper application
  * and treat as local file afterwards.
@@ -725,7 +714,6 @@ static FD_t ufdOpen(const char * url, int flags, mode_t mode)
 {
     FD_t fd = NULL;
     const char * path;
-    int timeout = 1;
     urltype urlType = urlPath(url, &path);
 
 if (_rpmio_debug)
@@ -747,7 +735,6 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
 	    fd = fdDup((flags & O_ACCMODE) == O_WRONLY ?
 			STDOUT_FILENO : STDIN_FILENO);
 	}
-	timeout = 600; /* XXX W2DO? 10 mins? */
 	break;
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
@@ -759,12 +746,11 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
     if (fd == NULL) return NULL;
 
     fdSetIo(fd, ufdio);
-    fd->rd_timeoutsecs = timeout;
     fd->bytesRemain = -1;
     fd->urlType = urlType;
 
     if (Fileno(fd) < 0) {
-	(void) ufdClose(fd);
+	(void) fdClose(fd);
 	return NULL;
     }
 DBGIO(fd, (stderr, "==>\tufdOpen(\"%s\",%x,0%o) %s\n", url, (unsigned)flags, (unsigned)mode, fdbg(fd)));
@@ -772,7 +758,7 @@ DBGIO(fd, (stderr, "==>\tufdOpen(\"%s\",%x,0%o) %s\n", url, (unsigned)flags, (un
 }
 
 static const struct FDIO_s ufdio_s = {
-  ufdRead, ufdWrite, ufdSeek, ufdClose, fdLink, fdFree, fdNew, fdFileno,
+  fdRead, fdWrite, fdSeek, fdClose, fdLink, fdFree, fdNew, fdFileno,
   ufdOpen, NULL, fdGetFp, NULL
 };
 static const FDIO_t ufdio = &ufdio_s ;
@@ -789,21 +775,9 @@ ssize_t timedRead(FD_t fd, void * bufptr, size_t length)
 
 #include <zlib.h>
 
-static inline void * gzdFileno(FD_t fd)
+static void * gzdFileno(FD_t fd)
 {
-    void * rc = NULL;
-    int i;
-
-    FDSANE(fd);
-    for (i = fd->nfps; i >= 0; i--) {
-	FDSTACK_t * fps = &fd->fps[i];
-	if (fps->io != gzdio)
-	    continue;
-	rc = fps->fp;
-	break;
-    }
-    
-    return rc;
+    return iotFileno(fd, gzdio);
 }
 
 static
@@ -870,7 +844,7 @@ DBGIO(fd, (stderr, "==>\tgzdRead(%p,%p,%u) rc %lx %s\n", cookie, buf, (unsigned)
 	}
     } else if (rc >= 0) {
 	fdstat_exit(fd, FDSTAT_READ, rc);
-	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
+	if (fd->digests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
     }
     return rc;
 }
@@ -883,7 +857,7 @@ static ssize_t gzdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd == NULL || fd->bytesRemain == 0) return 0;	/* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
+    if (fd->digests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     gzfile = gzdFileno(fd);
     if (gzfile == NULL) return -2;	/* XXX can't happen */
@@ -905,7 +879,7 @@ DBGIO(fd, (stderr, "==>\tgzdWrite(%p,%p,%u) rc %lx %s\n", cookie, buf, (unsigned
 }
 
 /* XXX zlib-1.0.4 has not */
-static inline int gzdSeek(void * cookie, _libio_pos_t pos, int whence)
+static int gzdSeek(void * cookie, _libio_pos_t pos, int whence)
 {
 #ifdef USE_COOKIE_SEEK_POINTER
     _IO_off64_t p = *pos;
@@ -1002,21 +976,9 @@ static const FDIO_t gzdio = &gzdio_s ;
 # define bzwrite BZ2_bzwrite
 #endif /* HAVE_BZ2_1_0 */
 
-static inline void * bzdFileno(FD_t fd)
+static void * bzdFileno(FD_t fd)
 {
-    void * rc = NULL;
-    int i;
-
-    FDSANE(fd);
-    for (i = fd->nfps; i >= 0; i--) {
-	FDSTACK_t * fps = &fd->fps[i];
-	if (fps->io != bzdio)
-	    continue;
-	rc = fps->fp;
-	break;
-    }
-    
-    return rc;
+    return iotFileno(fd, bzdio);
 }
 
 static FD_t bzdOpen(const char * path, const char * mode)
@@ -1071,7 +1033,7 @@ static ssize_t bzdRead(void * cookie, char * buf, size_t count)
 	    fd->errcookie = bzerror(bzfile, &zerror);
     } else if (rc >= 0) {
 	fdstat_exit(fd, FDSTAT_READ, rc);
-	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
+	if (fd->digests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
     }
     return rc;
 }
@@ -1084,7 +1046,7 @@ static ssize_t bzdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd->bytesRemain == 0) return 0;	/* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
+    if (fd->digests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     bzfile = bzdFileno(fd);
     fdstat_enter(fd, FDSTAT_WRITE);
@@ -1098,15 +1060,6 @@ static ssize_t bzdWrite(void * cookie, const char * buf, size_t count)
     return rc;
 }
 
-static inline int bzdSeek(void * cookie, _libio_pos_t pos,
-			int whence)
-{
-    FD_t fd = c2f(cookie);
-
-    BZDONLY(fd);
-    return -2;
-}
-
 static int bzdClose( void * cookie)
 {
     FD_t fd = c2f(cookie);
@@ -1141,7 +1094,7 @@ DBGIO(fd, (stderr, "==>\tbzdClose(%p) rc %lx %s\n", cookie, (unsigned long)rc, f
 }
 
 static const struct FDIO_s bzdio_s = {
-  bzdRead, bzdWrite, bzdSeek, bzdClose, fdLink, fdFree, fdNew, fdFileno,
+  bzdRead, bzdWrite, fdSeekNot, bzdClose, fdLink, fdFree, fdNew, fdFileno,
   NULL, bzdOpen, bzdFileno, bzdFlush
 };
 static const FDIO_t bzdio = &bzdio_s ;
@@ -1368,20 +1321,17 @@ static ssize_t lzwrite(LZFILE *lzfile, void *buf, size_t len)
 
 /* =============================================================== */
 
-static inline void * lzdFileno(FD_t fd)
+static void * lzdFileno(FD_t fd)
 {
     void * rc = NULL;
-    int i;
-
     FDSANE(fd);
-    for (i = fd->nfps; i >= 0; i--) {
-	    FDSTACK_t * fps = &fd->fps[i];
-	    if (fps->io != xzdio && fps->io != lzdio)
-		continue;
-	    rc = fps->fp;
+    for (int i = fd->nfps; i >= 0; i--) {
+	FDSTACK_t * fps = &fd->fps[i];
+	if (fps->io != xzdio && fps->io != lzdio)
+	    continue;
+	rc = fps->fp;
 	break;
     }
-    
     return rc;
 }
 
@@ -1460,7 +1410,7 @@ static ssize_t lzdRead(void * cookie, char * buf, size_t count)
 	fd->errcookie = "Lzma: decoding error";
     } else if (rc >= 0) {
 	fdstat_exit(fd, FDSTAT_READ, rc);
-	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
+	if (fd->digests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
     }
     return rc;
 }
@@ -1473,7 +1423,7 @@ static ssize_t lzdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd == NULL || fd->bytesRemain == 0) return 0;   /* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
+    if (fd->digests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     lzfile = lzdFileno(fd);
 
@@ -1487,14 +1437,6 @@ static ssize_t lzdWrite(void * cookie, const char * buf, size_t count)
     return rc;
 }
 
-static inline int lzdSeek(void * cookie, _libio_pos_t pos, int whence)
-{
-    FD_t fd = c2f(cookie);
-
-    LZDONLY(fd);
-    return -2;
-}
-
 static int lzdClose(void * cookie)
 {
     FD_t fd = c2f(cookie);
@@ -1526,13 +1468,13 @@ DBGIO(fd, (stderr, "==>\tlzdClose(%p) rc %lx %s\n", cookie, (unsigned long)rc, f
 }
 
 static struct FDIO_s xzdio_s = {
-  lzdRead, lzdWrite, lzdSeek, lzdClose, NULL, NULL, NULL, fdFileno,
+  lzdRead, lzdWrite, fdSeekNot, lzdClose, NULL, NULL, NULL, fdFileno,
   NULL, xzdOpen, lzdFileno, lzdFlush
 };
 static const FDIO_t xzdio = &xzdio_s;
 
 static struct FDIO_s lzdio_s = {
-  lzdRead, lzdWrite, lzdSeek, lzdClose, NULL, NULL, NULL, fdFileno,
+  lzdRead, lzdWrite, fdSeekNot, lzdClose, NULL, NULL, NULL, fdFileno,
   NULL, lzdOpen, lzdFileno, lzdFlush
 };
 static const FDIO_t lzdio = &lzdio_s;
@@ -1642,7 +1584,7 @@ DBGIO(fd, (stderr, "==> Fclose(%p) %s\n", (fd ? fd : NULL), fdbg(fd)));
 	    }
 	} else {
 	    fdio_close_function_t _close = FDIOVEC(fd, close);
-	    rc = _close(fd);
+	    rc = _close ? _close(fd) : -2;
 	}
 	if (fd->nfps == 0)
 	    break;
@@ -1665,7 +1607,7 @@ DBGIO(fd, (stderr, "==> Fclose(%p) %s\n", (fd ? fd : NULL), fdbg(fd)));
  * - bzopen:	's' is smallmode
  * - HACK:	'.' terminates, rest is type of I/O
  */
-static inline void cvtfmode (const char *m,
+static void cvtfmode (const char *m,
 				char *stdio, size_t nstdio,
 				char *other, size_t nother,
 				const char **end, int * f)
@@ -1759,29 +1701,29 @@ fprintf(stderr, "*** Fdopen(%p,%s) %s\n", fd, fmode, fdbg(fd));
 	return fd;
 
     if (end && *end) {
-	if (!strcmp(end, "fdio")) {
+	if (rstreq(end, "fdio")) {
 	    iof = fdio;
 #if HAVE_ZLIB_H
-	} else if (!strcmp(end, "gzdio")) {
+	} else if (rstreq(end, "gzdio") || rstreq(end, "gzip")) {
 	    iof = gzdio;
 	    fd = gzdFdopen(fd, zstdio);
 #endif
 #if HAVE_BZLIB_H
-	} else if (!strcmp(end, "bzdio")) {
+	} else if (rstreq(end, "bzdio") || rstreq(end, "bzip2")) {
 	    iof = bzdio;
 	    fd = bzdFdopen(fd, zstdio);
 #endif
 #if HAVE_LZMA_H
-	} else if (!strcmp(end, "xzdio")) {
+	} else if (rstreq(end, "xzdio") || rstreq(end, "xz")) {
 	    iof = xzdio;
 	    fd = xzdFdopen(fd, zstdio);
-	} else if (!strcmp(end, "lzdio")) {
+	} else if (rstreq(end, "lzdio") || rstreq(end, "lzma")) {
 	    iof = lzdio;
 	    fd = lzdFdopen(fd, zstdio);
 #endif
-	} else if (!strcmp(end, "ufdio")) {
+	} else if (rstreq(end, "ufdio")) {
 	    iof = ufdio;
-	} else if (!strcmp(end, "fpio")) {
+	} else if (rstreq(end, "fpio")) {
 	    iof = fpio;
 	    if (noLibio) {
 		int fdno = Fileno(fd);
@@ -1850,7 +1792,7 @@ FD_t Fopen(const char *path, const char *fmode)
     if (stdio[0] == '\0')
 	return NULL;
 
-    if (end == NULL || !strcmp(end, "fdio")) {
+    if (end == NULL || rstreq(end, "fdio")) {
 if (_rpmio_debug)
 fprintf(stderr, "*** Fopen fdio path %s fmode %s\n", path, fmode);
 	fd = fdOpen(path, flags, perms);
@@ -1913,6 +1855,21 @@ int Fflush(FD_t fd)
     return 0;
 }
 
+off_t Ftell(FD_t fd)
+{
+    FDIO_t iot;
+    off_t pos = -2; /* assume not implemented */
+
+    if (fd == NULL) return -1;
+    iot = fdGetIo(fd);
+    /* this wont work correctly for compressed types */
+    if (iot == fpio || iot == fdio || iot == ufdio) {
+	pos = lseek(Fileno(fd), 0, SEEK_CUR);
+    }
+
+    return pos;
+}
+
 int Ferror(FD_t fd)
 {
     int i, rc = 0;
@@ -2030,8 +1987,40 @@ exit:
 }
 
 static const struct FDIO_s fpio_s = {
-  ufdRead, ufdWrite, fdSeek, ufdClose, fdLink, fdFree, fdNew, fdFileno,
+  fdRead, fdWrite, fdSeek, fdClose, fdLink, fdFree, fdNew, fdFileno,
   ufdOpen, NULL, fdGetFp, NULL
 };
 static const FDIO_t fpio = &fpio_s ;
 
+void fdInitDigest(FD_t fd, pgpHashAlgo hashalgo, int flags)
+{
+    if (fd->digests == NULL) {
+	fd->digests = rpmDigestBundleNew();
+    }
+    fdstat_enter(fd, FDSTAT_DIGEST);
+    rpmDigestBundleAdd(fd->digests, hashalgo, flags);
+    fdstat_exit(fd, FDSTAT_DIGEST, (ssize_t) 0);
+}
+
+static void fdUpdateDigests(FD_t fd, const unsigned char * buf, size_t buflen)
+{
+    if (fd && fd->digests) {
+	fdstat_enter(fd, FDSTAT_DIGEST);
+	rpmDigestBundleUpdate(fd->digests, buf, buflen);
+	fdstat_exit(fd, FDSTAT_DIGEST, (ssize_t) buflen);
+    }
+}
+
+void fdFiniDigest(FD_t fd, pgpHashAlgo hashalgo,
+		void ** datap,
+		size_t * lenp,
+		int asAscii)
+{
+    if (fd && fd->digests) {
+	fdstat_enter(fd, FDSTAT_DIGEST);
+	rpmDigestBundleFinal(fd->digests, hashalgo, datap, lenp, asAscii);
+	fdstat_exit(fd, FDSTAT_DIGEST, (ssize_t) 0);
+    }
+}
+
+
