@@ -23,6 +23,7 @@ extern "C" {
 /** \ingroup rpmpgp
  */
 typedef struct DIGEST_CTX_s * DIGEST_CTX;
+typedef struct rpmDigestBundle_s * rpmDigestBundle;
 
 /** \ingroup rpmpgp
  */
@@ -1011,6 +1012,15 @@ int pgpPrtPkts(const uint8_t *pkts, size_t pktlen, pgpDig dig, int printing);
  */
 pgpArmor pgpReadPkts(const char * fn, uint8_t ** pkt, size_t * pktlen);
 
+/** \ingroup rpmpgp
+ * Parse armored OpenPGP packets from memory.
+ * @param armor		armored OpenPGP packet string
+ * @retval pkt		dearmored OpenPGP packet(s) (malloced)
+ * @retval pktlen	dearmored OpenPGP packet(s) length in bytes
+ * @return		type of armor found
+ */
+pgpArmor pgpParsePkts(const char *armor, uint8_t ** pkt, size_t * pktlen);
+
 /** \ingroup rpmpgp
  * Wrap a OpenPGP packets in ascii armor for transport.
  * @param atype		type of armor
@@ -1021,7 +1031,7 @@ pgpArmor pgpReadPkts(const char * fn, uint8_t ** pkt, size_t * pktlen);
 char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns);
 
 /** \ingroup rpmpgp
- * Create a container for parsed OpenPGP packates.
+ * Create a container for parsed OpenPGP packet(s).
  * @return		container
  */
 pgpDig pgpNewDig(void);
@@ -1033,12 +1043,27 @@ pgpDig pgpNewDig(void);
 void pgpCleanDig(pgpDig dig);
 
 /** \ingroup rpmpgp
- * Destroy a container for parsed OpenPGP packates.
+ * Destroy a container for parsed OpenPGP packet(s).
  * @param dig		container
  * @return		NULL always
  */
 pgpDig pgpFreeDig(pgpDig dig);
 
+/** \ingroup rpmpgp
+ * Verify a PGP signature.
+ * @param dig		container
+ * @param hashctx	digest context
+ * @return 		RPMRC_OK on success 
+ */
+rpmRC pgpVerifySig(pgpDig dig, DIGEST_CTX hashctx);
+
+/** \ingroup rpmpgp
+ * Return a string identification of a PGP signature/pubkey.
+ * @param digp		signature/pubkey container
+ * @return		string describing the item and parameters
+ */
+char *pgpIdentItem(pgpDigParams digp);
+
 /** \ingroup rpmpgp
  * Perform cryptography initialization.
  * It must be called before any cryptography can be used within rpm.
@@ -1100,6 +1125,59 @@ int rpmDigestFinal(DIGEST_CTX ctx,
 	void ** datap,
 	size_t * lenp, int asAscii);
 
+/** \ingroup rpmpgp
+ * Create a new digest bundle.
+ * @return		New digest bundle
+ */
+rpmDigestBundle rpmDigestBundleNew(void);
+
+/** \ingroup rpmpgp
+ * Free a digest bundle and all contained digest contexts.
+ * @param bundle	digest bundle
+ * @return		NULL always
+ */
+rpmDigestBundle rpmDigestBundleFree(rpmDigestBundle bundle);
+
+/** \ingroup rpmpgp
+ * Add a new type of digest to a bundle.
+ * @param bundle	digest bundle
+ * @param algo		type of digest
+ * @param flags		bit(s) to control digest operation
+ * @return		0 on success
+ */
+int rpmDigestBundleAdd(rpmDigestBundle bundle, pgpHashAlgo algo,
+			rpmDigestFlags flags);
+
+/** \ingroup rpmpgp
+ * Update contexts within bundle with next plain text buffer.
+ * @param bundle	digest bundle
+ * @param data		next data buffer
+ * @param len		no. bytes of data
+ * @return		0 on success
+ */
+int rpmDigestBundleUpdate(rpmDigestBundle bundle, const void *data, size_t len);
+
+/** \ingroup rpmpgp
+ * Return digest from a bundle and destroy context, see rpmDigestFinal().
+ *
+ * @param bundle	digest bundle
+ * @param algo		type of digest to return
+ * @retval datap	address of returned digest
+ * @retval lenp		address of digest length
+ * @param asAscii	return digest as ascii string?
+ * @return		0 on success
+ */
+int rpmDigestBundleFinal(rpmDigestBundle bundle,
+	 pgpHashAlgo algo, void ** datap, size_t * lenp, int asAscii);
+
+/** \ingroup rpmpgp
+ * Duplicate a digest context from a bundle.
+ * @param bundle	digest bundle
+ * @param algo		type of digest to dup
+ * @return		duplicated digest context
+ */
+DIGEST_CTX rpmDigestBundleDupCtx(rpmDigestBundle bundle, pgpHashAlgo algo);
+
 #ifdef __cplusplus
 }
 #endif
